@@ -1,11 +1,12 @@
 """
 This module provides decorators for various purposes:
 
-- silent(): Make a function silent (disable stdout, and stderr if specified) (alternative to stouputils.ctx.Muffle)
 - measure_time(): Measure the execution time of a function and print it with the given print function
 - handle_error(): Handle an error with different log levels
 - simple_cache(): Easy cache function with parameter caching method
-- deprecated(): Mark a function as deprecated
+- abstract(): Mark a function as abstract, using LogLevels for error handling
+- deprecated(): Mark a function as deprecated, using LogLevels for warning handling
+- silent(): Make a function silent (disable stdout, and stderr if specified) (alternative to stouputils.ctx.Muffle)
 
 .. image:: https://raw.githubusercontent.com/Stoupy51/stouputils/refs/heads/main/assets/decorators_module_1.gif
   :alt: stouputils decorators examples
@@ -27,66 +28,6 @@ from .ctx import MeasureTime, Muffle
 from .print import debug, error, warning
 
 
-def get_func_name(func: Callable[..., Any]) -> str:
-	try:
-		return func.__name__
-	except AttributeError:
-		return "<unknown>"
-
-def get_wrapper_name(decorator_name: str, func: Callable[..., Any]) -> str:
-	""" Get a descriptive name for a wrapper function.
-
-	Args:
-		decorator_name	(str):					Name of the decorator
-		func			(Callable[..., Any]):	Function being decorated
-	Returns:
-		str: Combined name for the wrapper function
-	"""
-	return f"{decorator_name}_{get_func_name(func)}"
-
-# Decorator that make a function silent (disable stdout)
-def silent(
-	func: Callable[..., Any] | None = None,
-	*,
-	mute_stderr: bool = False
-) -> Callable[..., Any]:
-	""" Decorator that makes a function silent (disable stdout, and stderr if specified).
-
-	Alternative to stouputils.ctx.Muffle.
-
-	Args:
-		func			(Callable[..., Any] | None):	Function to make silent
-		mute_stderr		(bool):							Whether to mute stderr or not
-
-	Examples:
-		>>> @silent
-		... def test():
-		...     print("Hello, world!")
-		>>> test()
-
-		>>> @silent(mute_stderr=True)
-		... def test2():
-		...     print("Hello, world!")
-		>>> test2()
-
-		>>> silent(print)("Hello, world!")
-	"""
-	def decorator(func: Callable[..., Any]) -> Callable[..., Any]:
-		@wraps(func)
-		def wrapper(*args: tuple[Any, ...], **kwargs: dict[str, Any]) -> Any:
-			# Use Muffle context manager to silence output
-			with Muffle(mute_stderr=mute_stderr):
-				return func(*args, **kwargs)
-		wrapper.__name__ = get_wrapper_name("stouputils.decorators.silent", func)
-		return wrapper
-
-	# Handle both @silent and @silent(mute_stderr=...)
-	if func is None:
-		return decorator
-	return decorator(func)
-
-
-
 # Execution time decorator
 def measure_time(
 	print_func: Callable[..., None] = debug,
@@ -118,7 +59,7 @@ def measure_time(
 	def decorator(func: Callable[..., Any]) -> Callable[..., Any]:
 
 		# Set the message if not specified, else use the provided one
-		new_msg: str = message if message else f"Execution time of {get_func_name(func)}"
+		new_msg: str = message if message else f"Execution time of {_get_func_name(func)}"
 
 		if is_generator:
 			@wraps(func)
@@ -130,12 +71,10 @@ def measure_time(
 			def wrapper(*args: tuple[Any, ...], **kwargs: dict[str, Any]) -> Any:
 				with MeasureTime(print_func=print_func, message=new_msg, perf_counter=perf_counter):
 					return func(*args, **kwargs)
-		wrapper.__name__ = get_wrapper_name("stouputils.decorators.measure_time", func)
+		wrapper.__name__ = _get_wrapper_name("stouputils.decorators.measure_time", func)
 		return wrapper
 	return decorator
 
-
-
 # Decorator that handle an error with different log levels
 class LogLevels(Enum):
 	""" Log level for the errors in the decorator handle_error() """
@@ -201,18 +140,18 @@ def handle_error(
 				return func(*args, **kwargs)
 			except exceptions as e:
 				if error_log == LogLevels.WARNING:
-					warning(f"{msg}Error during {get_func_name(func)}: ({type(e).__name__}) {e}")
+					warning(f"{msg}Error during {_get_func_name(func)}: ({type(e).__name__}) {e}")
 				elif error_log == LogLevels.WARNING_TRACEBACK:
-					warning(f"{msg}Error during {get_func_name(func)}:\n{format_exc()}")
+					warning(f"{msg}Error during {_get_func_name(func)}:\n{format_exc()}")
 				elif error_log == LogLevels.ERROR_TRACEBACK:
-					error(f"{msg}Error during {get_func_name(func)}:\n{format_exc()}", exit=True)
+					error(f"{msg}Error during {_get_func_name(func)}:\n{format_exc()}", exit=True)
 				elif error_log == LogLevels.RAISE_EXCEPTION:
 					raise e
 
 				# Sleep for the specified time, only if the error_log is not ERROR_TRACEBACK (because it's blocking)
 				if sleep_time > 0.0 and error_log != LogLevels.ERROR_TRACEBACK:
 					time.sleep(sleep_time)
-		wrapper.__name__ = get_wrapper_name("stouputils.decorators.handle_error", func)
+		wrapper.__name__ = _get_wrapper_name("stouputils.decorators.handle_error", func)
 		return wrapper
 
 	# Handle both @handle_error and @handle_error(exceptions=..., message=..., error_log=...)
@@ -220,8 +159,6 @@ def handle_error(
 		return decorator
 	return decorator(func)
 
-
-
 # Easy cache function with parameter caching method
 def simple_cache(
 	func: Callable[..., Any] | None = None,
@@ -230,7 +167,7 @@ def simple_cache(
 ) -> Callable[..., Any]:
 	""" Decorator that caches the result of a function based on its arguments.
 
-	The str method is often faster than the pickle method (by a little).
+	The str method is often faster than the pickle method (by a little) but not as accurate with complex objects.
 
 	Args:
 		func   (Callable[..., Any] | None): Function to cache
@@ -279,7 +216,7 @@ def simple_cache(
 				return result
 
 		# Return the wrapper
-		wrapper.__name__ = get_wrapper_name("stouputils.decorators.simple_cache", func)
+		wrapper.__name__ = _get_wrapper_name("stouputils.decorators.simple_cache", func)
 		return wrapper
 
 	# Handle both @simple_cache and @simple_cache(method=...)
@@ -287,64 +224,7 @@ def simple_cache(
 		return decorator
 	return decorator(func)
 
-
-def deprecated(
-	func: Callable[..., Any] | None = None,
-	*,
-	message: str = "",
-	error_log: LogLevels = LogLevels.WARNING
-) -> Callable[..., Any]:
-	""" Decorator that marks a function as deprecated.
-
-	Args:
-		func        (Callable[..., Any] | None): Function to mark as deprecated
-		message     (str):                       Additional message to display with the deprecation warning
-		error_log   (LogLevels):                 Log level for the deprecation warning
-			LogLevels.NONE:              None
-			LogLevels.WARNING:           Show as warning
-			LogLevels.WARNING_TRACEBACK: Show as warning with traceback
-			LogLevels.ERROR_TRACEBACK:   Show as error with traceback
-			LogLevels.RAISE_EXCEPTION:   Raise exception
-	Returns:
-		Callable[..., Any]: Decorator that marks a function as deprecated
-
-	Examples:
-		>>> @deprecated
-		... def old_function():
-		...     pass
-
-		>>> @deprecated(message="Use 'new_function()' instead", error_log=LogLevels.WARNING)
-		... def another_old_function():
-		...     pass
-	"""
-	def decorator(func: Callable[..., Any]) -> Callable[..., Any]:
-		@wraps(func)
-		def wrapper(*args: tuple[Any, ...], **kwargs: dict[str, Any]) -> Any:
-			# Build deprecation message
-			msg: str = f"Function '{get_func_name(func)}()' is deprecated"
-			if message:
-				msg += f". {message}"
-
-			# Handle deprecation warning based on log level
-			if error_log == LogLevels.WARNING:
-				warning(msg)
-			elif error_log == LogLevels.WARNING_TRACEBACK:
-				warning(f"{msg}\n{format_exc()}")
-			elif error_log == LogLevels.ERROR_TRACEBACK:
-				error(f"{msg}\n{format_exc()}", exit=True)
-			elif error_log == LogLevels.RAISE_EXCEPTION:
-				raise DeprecationWarning(msg)
-
-			# Call the original function
-			return func(*args, **kwargs)
-		wrapper.__name__ = get_wrapper_name("stouputils.decorators.deprecated", func)
-		return wrapper
-
-	# Handle both @deprecated and @deprecated(message=..., error_log=...)
-	if func is None:
-		return decorator
-	return decorator(func)
-
+# Decorator that marks a function as abstract
 def abstract(
 	func: Callable[..., Any] | None = None,
 	*,
@@ -380,7 +260,7 @@ def abstract(
 		NotImplementedError: Function 'method' is abstract and must be implemented by a subclass
 	"""
 	def decorator(func: Callable[..., Any]) -> Callable[..., Any]:
-		message: str = f"Function '{get_func_name(func)}' is abstract and must be implemented by a subclass"
+		message: str = f"Function '{_get_func_name(func)}' is abstract and must be implemented by a subclass"
 		if not func.__doc__:
 			func.__doc__ = message
 
@@ -388,7 +268,7 @@ def abstract(
 		@handle_error(exceptions=NotImplementedError, error_log=error_log)
 		def wrapper(*args: tuple[Any, ...], **kwargs: dict[str, Any]) -> Any:
 			raise NotImplementedError(message)
-		wrapper.__name__ = get_wrapper_name("stouputils.decorators.abstract", func)
+		wrapper.__name__ = _get_wrapper_name("stouputils.decorators.abstract", func)
 		return wrapper
 
 	# Handle both @abstract and @abstract(error_log=...)
@@ -396,3 +276,123 @@ def abstract(
 		return decorator
 	return decorator(func)
 
+# Decorator that marks a function as deprecated
+def deprecated(
+	func: Callable[..., Any] | None = None,
+	*,
+	message: str = "",
+	error_log: LogLevels = LogLevels.WARNING
+) -> Callable[..., Any]:
+	""" Decorator that marks a function as deprecated.
+
+	Args:
+		func        (Callable[..., Any] | None): Function to mark as deprecated
+		message     (str):                       Additional message to display with the deprecation warning
+		error_log   (LogLevels):                 Log level for the deprecation warning
+			LogLevels.NONE:              None
+			LogLevels.WARNING:           Show as warning
+			LogLevels.WARNING_TRACEBACK: Show as warning with traceback
+			LogLevels.ERROR_TRACEBACK:   Show as error with traceback
+			LogLevels.RAISE_EXCEPTION:   Raise exception
+	Returns:
+		Callable[..., Any]: Decorator that marks a function as deprecated
+
+	Examples:
+		>>> @deprecated
+		... def old_function():
+		...     pass
+
+		>>> @deprecated(message="Use 'new_function()' instead", error_log=LogLevels.WARNING)
+		... def another_old_function():
+		...     pass
+	"""
+	def decorator(func: Callable[..., Any]) -> Callable[..., Any]:
+		@wraps(func)
+		def wrapper(*args: tuple[Any, ...], **kwargs: dict[str, Any]) -> Any:
+			# Build deprecation message
+			msg: str = f"Function '{_get_func_name(func)}()' is deprecated"
+			if message:
+				msg += f". {message}"
+
+			# Handle deprecation warning based on log level
+			if error_log == LogLevels.WARNING:
+				warning(msg)
+			elif error_log == LogLevels.WARNING_TRACEBACK:
+				warning(f"{msg}\n{format_exc()}")
+			elif error_log == LogLevels.ERROR_TRACEBACK:
+				error(f"{msg}\n{format_exc()}", exit=True)
+			elif error_log == LogLevels.RAISE_EXCEPTION:
+				raise DeprecationWarning(msg)
+
+			# Call the original function
+			return func(*args, **kwargs)
+		wrapper.__name__ = _get_wrapper_name("stouputils.decorators.deprecated", func)
+		return wrapper
+
+	# Handle both @deprecated and @deprecated(message=..., error_log=...)
+	if func is None:
+		return decorator
+	return decorator(func)
+
+# Decorator that make a function silent (disable stdout)
+def silent(
+	func: Callable[..., Any] | None = None,
+	*,
+	mute_stderr: bool = False
+) -> Callable[..., Any]:
+	""" Decorator that makes a function silent (disable stdout, and stderr if specified).
+
+	Alternative to stouputils.ctx.Muffle.
+
+	Args:
+		func			(Callable[..., Any] | None):	Function to make silent
+		mute_stderr		(bool):							Whether to mute stderr or not
+
+	Examples:
+		>>> @silent
+		... def test():
+		...     print("Hello, world!")
+		>>> test()
+
+		>>> @silent(mute_stderr=True)
+		... def test2():
+		...     print("Hello, world!")
+		>>> test2()
+
+		>>> silent(print)("Hello, world!")
+	"""
+	def decorator(func: Callable[..., Any]) -> Callable[..., Any]:
+		@wraps(func)
+		def wrapper(*args: tuple[Any, ...], **kwargs: dict[str, Any]) -> Any:
+			# Use Muffle context manager to silence output
+			with Muffle(mute_stderr=mute_stderr):
+				return func(*args, **kwargs)
+		wrapper.__name__ = _get_wrapper_name("stouputils.decorators.silent", func)
+		return wrapper
+
+	# Handle both @silent and @silent(mute_stderr=...)
+	if func is None:
+		return decorator
+	return decorator(func)
+
+
+
+# "Private" functions
+def _get_func_name(func: Callable[..., Any]) -> str:
+	""" Get the name of a function, returns "<unknown>" if the name cannot be retrieved. """
+	try:
+		return func.__name__
+	except AttributeError:
+		return "<unknown>"
+
+def _get_wrapper_name(decorator_name: str, func: Callable[..., Any]) -> str:
+	""" Get a descriptive name for a wrapper function.
+
+	Args:
+		decorator_name	(str):					Name of the decorator
+		func			(Callable[..., Any]):	Function being decorated
+	Returns:
+		str: Combined name for the wrapper function (e.g., "decorator_name@function_name")
+	"""
+	return f"{decorator_name}@{_get_func_name(func)}"
+
